// -*- C++ -*-
/*!
 * @file SdoServiceConsumerBase.h
 * @brief SDO service consumer base class and its factory
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */


#ifndef RTC_SDOSERVICECONSUMERBASE_H
#define RTC_SDOSERVICECONSUMERBASE_H

#include <coil/Mutex.h>
#include <coil/Factory.h>
#include <coil/Timer.h>
#include <rtm/RTObject.h>
#include <rtm/idl/SDOPackageStub.h>

namespace RTC
{
  /*!
   * @if jp
   *
   * @brief SdoServiceConsumer@NX
   *
   * SDOŒ`ĂSDOT[rX̃RV[}邽߂̊N
   * XBSDOT[rXɂ́AO񋟃T[rXRTC(SDO)ŗp
   * SDOT[rXRV[}ƁARTC(SDO)gSDOT[rX񋟂SDO
   * T[rXvoC_BׂĂSDOT[rXRV[}͂̊
   * NXpĎB
   *
   * ̃IuWFNg̃CtTCN͈ȉ̒ʂB
   *
   * -# IuWFNg͒ʏALIuWFNg (so, DLL) ƂăRpCE
   *    NB
   * -# }l[Wɑ΂ă[hƃW[֐ɂIu
   *    WFNgt@NgASdoServiceConsumerFactory ɑ΂ēo^
   *    Bo^̃L[ɂ̓T[rXC^[tF[X IFR (interface
   *    repository) ID pAɂT[rXʂB
   * -# Õc[ȂǂT[rXvoC_A^b`ꂽꍇAT[
   *    rXC^[tF[X IFR ID ł SDO RV[}C
   *    X^XA񋟂ꂽSDOT[rX ServiceProfile (
   *    \̂̓T[rX̃IuWFNgt@X܂) RV[
   *    }ɃA^b`B
   * -# ̂Ƃ̃A^b`V[PX͈ȉ̒ʂB
   *   -# SDO::get_configuration() ɂ Configuration IuWFNg擾
   *   -# Configuration::add_service_profile() ɂO
   *      SdoServiceProvider  ServiceProfile ɂ RTC ɗ^B
   *   -# RTCŃT[rXĂяoKvLꍇA
   *      SdoServiceConsumer ێĂT[rXIuWFNgvLV
   *      ɑ΂ČĂяos
   * -# ŏII SdoServiceConsumer svɂȂꍇɂ́A
   *     Configuration::remove_service_profile()  id ƂƂɌĂяo
   *     SDOT[rXRV[} RTC 폜B
   *
   * <pre>
   * 
   *   [RTC] [SDO consumer] [Configuration]  [SDO service]    [Other]
   *     |          :             |                 |            |
   *     |          :         get_configuration()   |            |
   *     |<---------:-------------------------------|------------|
   *     |          :             |                 |            |
   *     |          :             |   add_service_profile(prof)  |
   *     |          :  create()   |<----------------|------------|
   *     |          |<------------|                 |            |
   *     |          |         call_sdo_service()    |            |
   *     |          |-------------|---------------->|            |
   *     |          |         call_sdo_service2()   |            |
   *     |          |-------------|---------------->|            |
   *     |          |             |       :         |            |
   *     |          |             |                 |            |
   *     |          |             | remove_service_profile(id)   |
   *     |          |  delete()   |<----------------|------------|
   *     |          x<------------|                 |            |
   *     |                        |                 x            x
   *
   * </pre>
   *
   * ̃NX̎ɓẮAȂƂȉ̏z֐
   * KvB
   *
   * - init(): ֐B^ꂽ RTObject  ServiceProfile 
   *   AYIuWFNgB
   * - reinit(): ď֐BServiceProfile ͐ݒXV̂ߓ
   *   IDŌĂяo邱ƂL邪A̍ۂɂ̊֐V
   *   ServiceProfile ƂƂɌĂяoB֐ł́Aݒ̕ύXȂ
   *   ďB
   * - getProfile(): ݒ肳ꂽvt@CԂ֐B
   * - finalize(): IBRV[}f^b`ۂɌĂяo
   *   ֐B֐ł͏IB
   *
   * SdoServiceConsumer ͒ʏ틤LIuWFNgƂăRpCEN
   * BLIuWFNg̃Gg|Cg͒ʏRpCꂽt@
   * C basename + "Init" ɂĂBȉɁANXAt@C
   * AGg|Cg֐̐B
   *
   * - NX: MySdoServiceConusmer 
   * - t@C: MySdoServiceConsumer.h. MySdoServiceConsumer.cpp
   * - LIuWFNg: MySdoServiceConsumer.so (or DLL)
   * - Gg|Cg֐: MySdoServiceConsumerInit()
   *
   * Gg|Cg֐͒ʏȉ̂悤ɁASdoServiceConsumerFactory
   * ɓYRV[}̃t@Ng (Ɖ̃t@N^) o^ȉ
   * 悤Ȋ֐ɂȂB
   *
   * <pre>
   * extern "C"
   * {
   *   void MySdoServiceConsumerInit()
   *   {
   *     RTC::SdoServiceConsumerFactory& factory
   *       = RTC::SdoServiceConsumerFactory::instance();
   *     factory.addFactory(CORBA_Util::toRepositoryId<OpenRTM::MySdoService>(),
   *                        ::coil::Creator< ::RTC::SdoServiceConsumerBase,
   *                        ::RTC::MySdoServiceConsumer>,
   *                        ::coil::Destructor< ::RTC::SdoServiceConsumerBase,
   *                        ::RTC::MySdoServiceConsumer>);
   *   }
   * };
   * </pre>
   * 
   * @else
   *
   * @endif
   *
   */
  class SdoServiceConsumerBase
  {
  public:
    /*!
     * @if jp
     * @brief zfXgN^
     * @else
     * @brief virtual destructor
     * @endif
     */
    virtual ~SdoServiceConsumerBase() {};

    /*!
     * @if jp
     * @brief RV[}NX̏֐
     *
     * ̃IuWFNg̏sBOSDOT[rX
     * ServiceProfile ƂƂɃA^b`ƁASDORV[}CX
     * ^XA̒ SDO T[rXA^b`ꂽ RTC Ɨ^
     * ꂽ ServiceProfile ƂĂ̊֐Ă΂B
     *
     * ֐ł́AServiceProfile  SDO T[rXt@X
     * CorbaConsumer NX𗘗pIuWFNgɕێƂƂɁA
     * properties ݒeǂݍ݃T[rXŗL̐ݒ蓙sB^
     * ꂽT[rX̃IuWFNgt@XsA邢
     * properties ̓esȀꍇ͖߂l false ԂB
     *
     * @param rtobj ̃IuWFNgCX^Xꂽ RTC
     * @param profile O^ꂽ SDO ServiceProfile
     * @return ^ꂽ SDO Service  ServiceProfile s̏ꍇ false
     *
     * @else
     * @brief Initialization function of the consumer class
     *
     * @endif
     */
    virtual bool init(RTObject_impl& rtobj,
                      const SDOPackage::ServiceProfile& profile) = 0;
    /*!
     * @if jp
     * @brief RV[}NX̍ď֐
     *
     * ̃IuWFNg̍ďsBServiceProfile ɂ id tB[
     * hɃZbVŗL UUID ZbgĂ邪A id ̏
     * Aproperties ɐݒ肳ꂽݒ̕ύXAservice tB[h
     * ̃T[rX̎QƂ̕ύXsB̍ۂɌĂ΂̂
     * reinit() ֐łBł́Aservice tB[h̃IuWFNg
     * t@X̓ꐫmFAقȂĂꍇێĂ郊t@
     * XXVKvB܂ properties ɂ͐VȐݒ肪^
     * Ă\̂ŁAeǂݍݐݒXVB
     *
     * @param profile Vɗ^ꂽ SDO ServiceProfile
     * @return s ServiceProfile ^ꂽꍇ false
     *
     * @else
     * @brief Reinitialization function of the consumer class
     *
     * @endif
     */
    virtual bool reinit(const SDOPackage::ServiceProfile& profile) = 0;

    /*!
     * @if jp
     * @brief ServiceProfile Ԃ
     *
     * init()/reinit()ŗ^ꂽ ServiceProfile ͒ʏIuWFNg
     * ŕێBSDO Service Ǘt[[N͊Ǘケ̃IuWF
     * NgɑΉ ServiceProfile KvƂ̂ŁÅ֐ł͕ێ
     * Ă ServiceProfile ԂB
     * 
     * @return ̃IuWFNgێĂ ServiceProfile
     *
     * @else
     * @brief Getting ServiceProfile
     * @endif
     */
    virtual const SDOPackage::ServiceProfile& getProfile() const = 0;

    /*!
     * @if jp
     * @brief I
     *
     * SDOT[rXŃ^b`ۂɌĂяoIp֐BT[
     * rX̂Ń^b`ɍۂāAYIuWFNgێ郊\[X
     * Ȃǂ̏sB
     *
     * @else
     * @brief Finalization
     *
     * @endif
     */
    virtual void finalize() = 0;
  };

    /*!
     * @if jp
     * @brief SdoServiceConsumerFactory  typedef
     * @else
     * @brief typedef of sdoServiceConsumerFactory
     * @endif
     */
  typedef ::coil::GlobalFactory<
    ::RTC::SdoServiceConsumerBase > SdoServiceConsumerFactory;

#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
    /*!
     * @if jp
     * @brief NXev[g̖ICX^X
     * @else
     * @brief Explicit instantiation of class template
     * @endif
     */
  EXTERN template class DLL_PLUGIN 
                     ::coil::GlobalFactory< ::RTC::SdoServiceConsumerBase >;
#endif  
}; // namespace RTC

#endif // RTC_SDOSERVICECONSUMERBASE_H
