// -*- C++ -*-
/*!
 * @file SdoServiceProviderBase.h
 * @brief SDO service provider base class and its factory
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */


#ifndef RTC_SDOSERVICEPROVIDERBASE_H
#define RTC_SDOSERVICEPROVIDERBASE_H

#include <coil/Mutex.h>
#include <coil/Factory.h>
#include <coil/Timer.h>
#include <rtm/RTC.h>
#include <rtm/RTObject.h>
#include <rtm/idl/SDOPackageStub.h>

namespace RTC
{
  /*!
   * @if jp
   *
   * @brief SdoServiceProvider@NX
   *
   * SDOŒ`ĂSDOT[rX̃voC_邽߂̊N
   * XBSDOT[rXɂ́AO񋟃T[rXRTC(SDO)ŗp
   * SDOT[rXRV[}ƁARTC(SDO)gSDOT[rX񋟂SDO
   * T[rXvoC_BׂĂSDOT[rXvoC_͂̊
   * NXpĎB
   *
   * ̃IuWFNg̃CtTCN͈ȉ̒ʂB
   *
   * -# IuWFNg͒ʏALIuWFNg (so, DLL) ƂăRpCE
   *    NB
   * -# }l[Wɑ΂ă[hƃW[֐ɂIu
   *    WFNgt@NgASdoServiceProviderFactory ɑ΂ēo^
   *    Bo^̃L[ɂ̓T[rXC^[tF[X IFR (interface
   *    repository) ID pAɂT[rXʂB
   * -# rtc.conf̃RtBM[VwɂAL邱Ƃ
   *    w肳ĂT[rXCvoC_́ARTC̋NƓɃCX
   *    ^XB
   * -# CX^XA֐ init() Ă΂Bɂ͓YT[
   *    rX̂߂̃RtBM[VIvV coil::Property
   *    nB
   * -# CX^XꂽSDOT[rXvoC_
   *    SDO::get_get_sdo_service() ɂOANZXB̂
   *    AT[rXw肷IDIFR IDƓłB̂Ƃ̃A^b
   *    `V[PX͈ȉ̒ʂB
   * -# RTCfinalizêƓSDOT[rXvoC_
   *    邪A̍ۂɂSdoServiceProviderBase::finalize()R[
   *    ̂ŁAŃ\[X̉ȂǏIsB
   *
   * <pre>
   * 
   *   [RTC]      [SDO service]               [Other]
   *     |              :                        |
   *     | instantiate  :                        |
   *     |------------->:                        |
   *     |    init()    |                        |
   *     |------------->|                        |
   *     |              | get_service_profiles() |
   *     |<--------------------------------------|
   *     |              |    get_sdo_service()   |
   *     |<--------------------------------------|
   *     |              |        use service     |
   *     |              |<-----------------------|
   *     |              |                        |
   *     |  finalize()  |                        |
   *     |------------->x                        |
   *     x              x                        |
   *
   * </pre>
   *
   * ̃NX̎ɓẮAȂƂȉ̏z֐
   * KvB
   *
   * - init(): ֐B^ꂽ RTObject  ServiceProfile 
   *   AYIuWFNgB
   * - reinit(): ď֐BServiceProfile ͐ݒXV̂ߓ
   *   IDŌĂяo邱ƂL邪A̍ۂɂ̊֐V
   *   ServiceProfile ƂƂɌĂяoB֐ł́Aݒ̕ύXȂ
   *   ďB
   * - getProfile(): ݒ肳ꂽvt@CԂ֐B
   * - finalize(): IBRV[}f^b`ۂɌĂяo
   *   ֐B֐ł͏IB
   *
   * SdoServiceProvider ͒ʏ틤LIuWFNgƂăRpCEN
   * BLIuWFNg̃Gg|Cg͒ʏRpCꂽt@
   * C basename + "Init" ɂĂBȉɁANXAt@C
   * AGg|Cg֐̐B
   *
   * - NX: MySdoServiceConusmer 
   * - t@C: MySdoServiceProvider.h. MySdoServiceProvider.cpp
   * - LIuWFNg: MySdoServiceProvider.so (or DLL)
   * - Gg|Cg֐: MySdoServiceProviderInit()
   *
   * Gg|Cg֐͒ʏȉ̂悤ɁASdoServiceProviderFactory
   * ɓYRV[}̃t@Ng (Ɖ̃t@N^) o^ȉ
   * 悤Ȋ֐ɂȂB
   *
   * <pre>
   * extern "C"
   * {
   *   void MySdoServiceProviderInit()
   *   {
   *     RTC::SdoServiceProviderFactory& factory
   *       = RTC::SdoServiceProviderFactory::instance();
   *     factory.addFactory(CORBA_Util::toRepositoryId<OpenRTM::MySdoService>(),
   *                        ::coil::Creator< ::RTC::SdoServiceProviderBase,
   *                        ::RTC::MySdoServiceProvider>,
   *                        ::coil::Destructor< ::RTC::SdoServiceProviderBase,
   *                        ::RTC::MySdoServiceProvider>);
   *   }
   * };
   * </pre>
   * 
   * @else
   *
   * @endif
   *
   */
  class SdoServiceProviderBase
    : public virtual POA_SDOPackage::SDOService,
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief zfXgN^
     * @else
     * @brief virtual destructor
     * @endif
     */
    virtual ~SdoServiceProviderBase() {};

    /*!
     * @if jp
     * @brief RV[}NX̏֐
     *
     * ̃IuWFNg̏sBOSDOT[rX
     * ServiceProfile ƂƂɃA^b`ƁASDORV[}CX
     * ^XA̒ SDO T[rXA^b`ꂽ RTC Ɨ^
     * ꂽ ServiceProfile ƂĂ̊֐Ă΂B
     *
     * ֐ł́AServiceProfile  SDO T[rXt@X
     * CorbaProvider NX𗘗pIuWFNgɕێƂƂɁA
     * properties ݒeǂݍ݃T[rXŗL̐ݒ蓙sB^
     * ꂽT[rX̃IuWFNgt@XsA邢
     * properties ̓esȀꍇ͖߂l false ԂB
     *
     * @param rtobj ̃IuWFNgCX^Xꂽ RTC
     * @param profile O^ꂽ SDO ServiceProfile
     * @return ^ꂽ SDO Service  ServiceProfile s̏ꍇ false
     *
     * @else
     * @brief Initialization function of the consumer class
     *
     * @endif
     */
    virtual bool init(RTObject_impl& rtobj,
                      const SDOPackage::ServiceProfile& profile) = 0;
    /*!
     * @if jp
     * @brief RV[}NX̍ď֐
     *
     * ̃IuWFNg̍ďsBServiceProfile ɂ id tB[
     * hɃZbVŗL UUID ZbgĂ邪A id ̏
     * Aproperties ɐݒ肳ꂽݒ̕ύXAservice tB[h
     * ̃T[rX̎QƂ̕ύXsB̍ۂɌĂ΂̂
     * reinit() ֐łBł́Aservice tB[h̃IuWFNg
     * t@X̓ꐫmFAقȂĂꍇێĂ郊t@
     * XXVKvB܂ properties ɂ͐VȐݒ肪^
     * Ă\̂ŁAeǂݍݐݒXVB
     *
     * @param profile Vɗ^ꂽ SDO ServiceProfile
     * @return s ServiceProfile ^ꂽꍇ false
     *
     * @else
     * @brief Reinitialization function of the consumer class
     *
     * @endif
     */
    virtual bool reinit(const SDOPackage::ServiceProfile& profile) = 0;

    /*!
     * @if jp
     * @brief ServiceProfile Ԃ
     *
     * init()/reinit()ŗ^ꂽ ServiceProfile ͒ʏIuWFNg
     * ŕێBSDO Service Ǘt[[N͊Ǘケ̃IuWF
     * NgɑΉ ServiceProfile KvƂ̂ŁÅ֐ł͕ێ
     * Ă ServiceProfile ԂB
     * 
     * @return ̃IuWFNgێĂ ServiceProfile
     *
     * @else
     * @brief Getting ServiceProfile
     * @endif
     */
    virtual const SDOPackage::ServiceProfile& getProfile() const = 0;

    /*!
     * @if jp
     * @brief I
     *
     * SDOT[rXŃ^b`ۂɌĂяoIp֐BT[
     * rX̂Ń^b`ɍۂāAYIuWFNgێ郊\[X
     * Ȃǂ̏sB
     *
     * @else
     * @brief Finalization
     *
     * @endif
     */
    virtual void finalize() = 0;
  };

    /*!
     * @if jp
     * @brief SdoServiceProviderFactory  typedef
     * @else
     * @brief typedef of sdoServiceProviderFactory
     * @endif
     */
  typedef ::coil::GlobalFactory<
    ::RTC::SdoServiceProviderBase > SdoServiceProviderFactory;

#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
    /*!
     * @if jp
     * @brief NXev[g̖ICX^X
     * @else
     * @brief Explicit instantiation of class template
     * @endif
     */
  EXTERN template class DLL_PLUGIN 
                     ::coil::GlobalFactory< ::RTC::SdoServiceProviderBase >;
#endif  
}; // namespace RTC

#endif // RTC_SDOSERVICEPROVIDERBASE_H
